-------------------------------------------------------------------------------
--! @file       L_array.vhd
--! @author     William Diehl
--! @brief      OMD L_array
--! @date       29 July 2016
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;  
use ieee.std_logic_arith.all;


entity L_array is
generic (n : integer:=256;
			G_MAX_L_INDEX: std_logic_vector(4 downto 0):= '0' & x"3";
			G_INDEX_WIDTH: integer:= 3
);
port(
	clk		:in std_logic;
	rst		:in std_logic;
	start		:in std_logic;
	load		:in std_logic;
	done		:out std_logic;

	lstar 		:in std_logic_vector(n-1 downto 0);

	index		:in std_logic_vector(G_INDEX_WIDTH - 1 downto 0);
	l_index	        :out std_logic_vector(n-1 downto 0)
);
end L_array;

architecture structural of L_array is

signal  load_counter, next_load_counter :std_logic_vector(4 downto 0);
signal	addr :std_logic_vector(G_INDEX_WIDTH - 1 downto 0);
signal	l_first, l_reg, next_L, l_2, l_4 :std_logic_vector(n-1 downto 0);
begin

	next_load_counter <= load_counter + 1;
	addr <= load_counter(G_INDEX_WIDTH - 1 downto 0) when (load = '1') else index;
	l_first <= lstar when (load_counter = 0) else l_reg; 
	done <= '1' when (load_counter = G_MAX_L_INDEX) else '0';
		
	get_x2: entity work.GF2(dataflow)
     	port map (
    	x => l_first, 
    	o => l_2);

	get_x4: entity work.GF2(dataflow)
     	port map (
    	x => l_2, 
    	o => l_4);

	next_L <= l_4 when (load_counter = 0) else l_2;

-- store next L

	store_L: entity work.regn_en(behavioral)
	generic map(n=>256)
	port map(
		d => next_L,
		reset => rst,
		clk => clk,
		init => '0',
		en => load,
		q => l_reg);

-- L precomputed RAM

	L_precompute: entity work.L_DRAM(behavioral)
	generic map(n=> 256, addr_width=>G_INDEX_WIDTH)
	port map(
      		clk => clk,
				we => load,
				di => next_L,
				do => l_index,
				addr => addr
		);

-- update counter

	update_cnt: entity work.regn_en(behavioral)
	generic map(n=>5)
	port map(
		d => next_load_counter,
		reset => rst,
		clk => clk,
		init => start,
		en => load,
		q => load_counter);

end structural;

