-------------------------------------------------------------------------------
--! @file       sha256_round.vhd
--! @author     William Diehl
--! @brief      OMD sha256_round
--! @date       29 July 2016
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all; 
use ieee.std_logic_unsigned.all;  

entity sha256_round is
generic(n : integer :=32);
port(

	a		:in std_logic_vector(n-1 downto 0);
	b		:in std_logic_vector(n-1 downto 0);
	c		:in std_logic_vector(n-1 downto 0);
	d		:in std_logic_vector(n-1 downto 0);
	e		:in std_logic_vector(n-1 downto 0);
	f		:in std_logic_vector(n-1 downto 0);
	g		:in std_logic_vector(n-1 downto 0);
	h		:in std_logic_vector(n-1 downto 0);

	X		:in std_logic_vector(n-1 downto 0);
	K		:in std_logic_vector(n-1 downto 0);

	kHX_in		:in std_logic_vector(n-1 downto 0);
	kHX_out		:out std_logic_vector(n-1 downto 0);

	a_out		:out std_logic_vector(n-1 downto 0);
	b_out		:out std_logic_vector(n-1 downto 0);
	c_out		:out std_logic_vector(n-1 downto 0);
	d_out		:out std_logic_vector(n-1 downto 0);
	e_out		:out std_logic_vector(n-1 downto 0);
	f_out		:out std_logic_vector(n-1 downto 0);
	g_out		:out std_logic_vector(n-1 downto 0);
	h_out		:out std_logic_vector(n-1 downto 0));

end sha256_round;

architecture structural of sha256_round is

signal  T1, T2, chefg, majabc, sigma0a, sigma1e : std_logic_vector(n-1 downto 0);

begin

	a_out <= T1 + T2; 
	
	b_out <= a;
	c_out <= b;
	d_out <= c;
	e_out <= d + T1;
	
	f_out <= e;
	g_out <= f;
	h_out <= g;

	kHX_out <= g + k + X;

	T1 <= chefg + sigma1e + kHX_in;
	
	T2 <= sigma0a + majabc;

	get_Maj: entity work.Maj(dataflow)
     	generic map (n => 32)
     	port map (
    		x => a,
    		y => b,
    		z => c, 
    		o => majabc);

	get_Ch: entity work.Ch(dataflow)
     	generic map (n => 32)
     	port map (
    		x => e,
    		y => f,
    		z => g, 
    		o => chefg);

	get_Sigma0: entity work.Sigma0(dataflow)
     	generic map (n => 32)
     	port map (
    		x => a, 
    		o => sigma0a);

	get_Sigma1: entity work.Sigma1(dataflow)
     	generic map (n => 32)
     	port map (
    		x => e, 
    		o => sigma1e);

end structural; 

